@@ -68,7 +68,10 @@
 
 /* Numeric references to the dictionary items that need special handling.
    These must match the values in the dictionary! You can give them any 
-   names you want in the dictionary, as long as you use the values below. */
+   names you want in the dictionary, as long as you use the values below. 
+   
+   In addition to the main dictionary, these come from dict.internal and
+   dict.stdauth. */
 
 #define C_DS_GROUND		0	/* Ground space, holding whole packet */
 #define C_DS_RAD_PKT		1	/* Space that holds items below */
@@ -93,6 +96,10 @@
 #define C_DI_TIMESTAMP		10	/* Timestamp of request */
 #define C_DI_SECRET		11	/* Shared secret to sign response */
 #define C_DI_LOG_LINE		12	/* Line logged at end of expression */
+#define C_DI_SERVER_PID		13	/* Replaced by PID of server after
+					   processing configuration file if
+					   present on request list then */
+#define C_DI_REQUEST_NR		14	/* Incremented for each request */
 
 #define C_DI_IP_SOURCE		16	/* Source IP address of request */
 #define C_DI_IP_DEST		17	/* Destination address of request */
@@ -113,6 +120,10 @@
 #define C_DI_WINDOW		36	/* Config: TX window to subprograms */
 #define C_DI_JOBTICKET		37	/* Config: Attribute to put job 
 					   ticket in for asynchronous ifaces */
+#define C_DI_PIDATTR		38	/* Config: Attribute to put selected
+					   PID in for module interface, so
+					   that we can route related requests
+					   to the same module instance */
 
 /* Interface flag constants (C_DI_FLAGS) */
 
@@ -158,7 +169,7 @@
 #define C_DV_FLAGS_USEDBITS	26
 
 
-/* 
+/************************************************************************
  * The following constants are not used by the server itself, but are
  * intended for use by modules that use the binary interface and don't
  * read the dictionary.
@@ -180,8 +191,7 @@
 #define C_DI_USER_NAME		1	/* Used by eg. RadLDAP */
 #define C_DI_USER_PASSWORD	2	/* Used by eg. RadLDAP */
 
-
 /* Attributes for modules that pass a transaction ID around. In DS_INTERNAL */
 
-#define C_DI_TID		100	/* Used by eg. EAPTLS */
+#define C_DI_TID		50	/* Used by eg. EAPTLS */
 
