@@ -46,6 +46,8 @@
  * 2002/12/12 - EvB - Fixed incorrect behaviour of meta_getitemby*, which used
  * 		      META_ORD_ERR as a wildcard for vendor numbers; introduced
  * 		      META_VND_WILD for that purpose instead.
+ * 2005/06/15 - EvB - Added 'max_size' to specify maximum value size before
+ * 		      splitting (used by buildtree); calculated by additem
  */
 
 char metadata_id[] = "METADATA - Copyright (C) 2000 Emile van Bergen.";
@@ -361,6 +363,11 @@ META_ITEM *meta_additem(META *m, META_SPC *s,
 	ret->nodec = nodec; ret->noenc = noenc;
 	ret->keeprej = keeprej; ret->keepacct = keepacct;
 
+	/* Calculate max_size if not given as parameter (optional param tbd) */
+	ret->max_size = val_size;
+	if (val_size <= 0)
+	      ret->max_size = (1 << (len_size << 3)) - 1 + len_adj + val_size;
+
 	/* Add to list */
 	ret->next = s->items;
 	s->items = ret;
