@@ -40,6 +40,8 @@
  * 2004/07/15 - EvB - Made signed chars explicitly signed as they should be
  * 2005/06/15 - EvB - Added 'max_size' to specify maximum value size before
  * 		      splitting (used by buildtree)
+ * 2005/12/05 - EvB - Changed META_ORD to unsigned. Signed arithmetic is the
+ * 		      exception in the code and behaviour files, not the rule.
  */
 
 
@@ -65,15 +67,21 @@
    those fields can be handled as a normal ordinal, which greatly facilitates 
    common operations like searching etc. This type defines that host type. */
 
-typedef long META_ORD;
+typedef unsigned long META_ORD;
 
-#define META_ORD_ERR	((-LONG_MAX)-1)		/* not an integer number */
 
-#define META_VND_WILD	-1			/* specify a wildcard search */
-#define META_VND_ANY	META_ORD_ERR		/* specifically search for
-						   attributes allowed for any
+/* Some semaphore values - use these only in the contexts they're defined for, 
+ * that is, vendor- or attribute/item numbers. When META_ORD represents user
+ * or network input, these values are not allowed to get specific treatment! */
+
+#define META_ORD_ERR	((-LONG_MAX)-1)		/* eq. of -128 resp. -32768 */
+#define META_VND_WILD	(-LONG_MAX)		/* eq. of -127, specifies a
+						   search disregarding vendor */
+#define META_VND_ANY	META_ORD_ERR		/* search for attributes 
+						   specifically allowed for any
 						   vendor; attributes having
-						   vendor=x end up here. */
+						   vendor=x end up here.
+						   */
 
 
 /* Another fixed definition: names. They have a fixed length, so that
