@@ -89,7 +89,7 @@ typedef struct stdop {
 #define STDOP_CNT	85
 
 static STDOP stdops[STDOP_CNT] = {
-	/* Statements */
+	/* Statements (don't leave anything on stack) */
 	{ "halt",	 NON, 32, L,   ANY, NON,   INT, OP_HALT },
 	{ "abort",	 NON, 32, L,   ANY, NON,   INT, OP_ABORT },
 	{ "accept",	 NON, 32, L,   ANY, NON,   INT, OP_ACCEPT },
@@ -368,7 +368,7 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 
 			o->op = OP_PUSHINT; 
 			o->imm.ord = GETIP(&l);
-			context = IPA; ctx_item = 0;
+			context = IPA;
 			if (!l) { 
 				o->imm.ord = GETORD(0, &l);
 				context = INT;
@@ -395,7 +395,7 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 
 			/* On to the next insn and skip the displacement */
 			o += 1 + n; ret += (1 + n) * sizeof(INSN);
-			context = STR; ctx_item = 0;
+			context = STR;
 			continue;
 		}
 
@@ -492,7 +492,6 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 				o++; ret += sizeof(INSN); 
 
 				/* Done */
-				ctx_item = 0;
 				continue;
 			}
 		}
@@ -540,10 +539,9 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 				o->op = so->op; o++; ret += sizeof(INSN);
 			}
 
-			/* Set new context. Don't reset ctx_item, otherwise
-			   Flags = Ascii + Add-Spaces will work allright, but
-			   Flags = Ascii + Add-Spaces + Add-Tabs won't!!! */
+			/* Set new context; reset ctx_item if none (comma) */
 			context = so->newctx;
+			if (context == NON) ctx_item = 0;
 			continue;
 		}
 			
@@ -553,15 +551,14 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 		 */
 
 		/* Note that these are tested after the operators, unlike the
-		   normal terms to prevent unnecessary searches through the
+		   normal terms, to prevent unnecessary searches through the
 		   dictionary. */
 
 		/* Attribute reference or named value term */
 		if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
 
-			o->imm.i.flags = 0;
-
 			/* Process request or reply override and 'first' flag */
+			o->imm.i.flags = 0;
 			if (!NCMP("REQ:", 4)) {
 				S(4); o->imm.i.flags = AV_USEREPVALID;
 			}
@@ -573,12 +570,25 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 			}
 
 			/* Set l to length of identifier, leave colon or minus
-			   out if it's the last char, mostly for eg. 'int:=3' */
+			   out if it's the last char, for int:=3 and int-5 */
 			l = SPN("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 				"abcdefghijklmnopqrstuvwxyz0123456789:-");
 			if (I(l - 1) == ':' || I(l - 1) == '-') l--;
 
-			/* Search in dictionary item list */
+			/* If we have an attribute context in scope and 
+			 * there were no attribute reference flags given,
+			 * we search the constant value list first */
+
+			if (ctx_item && o->imm.i.flags == 0 &&
+			    (v = getvalbyspec(m, ctx_item, source, l))) {
+				S(l);
+				context = INT;
+				o->imm.ord = v->nr;
+				o->op = OP_PUSHINT; o++; ret += sizeof(INSN);
+				continue;
+			}
+
+			/* Not found; search in dictionary item list */
 			o->imm.i.item = getitembyspec(m, source, l);
 			if (o->imm.i.item) {
 				S(l);
@@ -596,16 +606,6 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 				continue;
 			}
 
-			/* Not found, try the constant value list */
-			if (ctx_item &&
-			    (v = getvalbyspec(m, ctx_item, source, l))) {
-				S(l);
-				context = INT;
-				o->imm.ord = v->nr;
-				o->op = OP_PUSHINT; o++; ret += sizeof(INSN);
-				continue;
-			}
-
 			/* Error. See if we have an itemized context or not. */
 			if (ctx_item) {
 				msg(F_LANG, L_ERR, "compile: ERROR: Unknown "
