@@ -31,7 +31,7 @@ char config_id[] = "CONFIG - Copyright (C) 2001 Emile van Bergen.";
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>	/* For memset() */
-#include <unistd.h>	/* For close() */
+#include <unistd.h>	/* For close(), getpid() */
 #include <netinet/in.h>	/* For struct in_addr */
 #include <arpa/inet.h>	/* For inet_ntoa() */
 #include <fcntl.h>
@@ -154,6 +154,7 @@ static int add_iface(CONF *c, META *m, META_AV *data,
 		  case C_DI_SENDATTR:
 		  case C_DI_RECVATTR:
 		  case C_DI_JOBTICKET:
+		  case C_DI_PIDATTR:
 			setspec_n(spec, av->p, av->l);
 			i = meta_getitembyspec(m, spec);
 			if (!i) { msg(F_MISC, L_ERR, "add_iface: ERROR: Unknown ACL/jobticket attribute '%s'!\n", spec); return -1; }
@@ -163,6 +164,11 @@ static int add_iface(CONF *c, META *m, META_AV *data,
 				iface->jobticket = i;
 				break;
 			}
+			if (av->i->nr == C_DI_PIDATTR) {
+				if (i->val_type != MT_INTEGER) { msg(F_MISC, L_ERR, "add_iface: ERROR: Module PID attribute '%s' must be of integer type!\n", spec); return -1; }
+				iface->pidattr = i;
+				break;
+			}
 
 			/* Allocate AV item */
 			aclav = (META_AV *)malloc(sizeof(META_AV));
@@ -253,6 +259,7 @@ static int add_iface(CONF *c, META *m, META_AV *data,
 
 CONF *conf_new(META *m, char *basepath, char *modbasepath)
 {
+	META_AV *av;
 	CONF *ret;
 	IFACE *ifs;
 	TEXT *t;
@@ -270,13 +277,14 @@ CONF *conf_new(META *m, char *basepath, char *modbasepath)
 	memset(ret, 0, sizeof(CONF));
 	ret->m = m;
 	if (!(ret->ds_ground = meta_getspcbynr(m, C_DS_GROUND)) ||
+	    /* RAD-PKT */
 	    !(ret->ds_rad_pkt = meta_getspcbynr(m, C_DS_RAD_PKT)) ||
-	    !(ret->ds_rad_atr = meta_getspcbynr(m, C_DS_RAD_ATR)) ||
 	    !(ret->di_code = meta_getitembynr(m, ret->ds_rad_pkt, 
 	    					 C_DI_CODE, META_VND_ANY)) ||
 	    !(ret->di_authenticator = meta_getitembynr(m, ret->ds_rad_pkt, 
 	    					 C_DI_AUTHENTICATOR, 
 						 META_VND_ANY)) ||
+	    /* INTERNAL */
 	    !(ret->ds_internal = meta_getspcbynr(m, C_DS_INTERNAL)) ||
 	    !(ret->di_timestamp = meta_getitembynr(m, ret->ds_internal, 
 	    					 C_DI_TIMESTAMP, 
@@ -305,6 +313,14 @@ CONF *conf_new(META *m, char *basepath, char *modbasepath)
 	    !(ret->di_log_line = meta_getitembynr(m, ret->ds_internal, 
 	    					 C_DI_LOG_LINE, 
 						 META_VND_ANY)) ||
+	    !(ret->di_server_pid = meta_getitembynr(m, ret->ds_internal, 
+	    					 C_DI_SERVER_PID, 
+						 META_VND_ANY)) ||
+	    !(ret->di_request_nr = meta_getitembynr(m, ret->ds_internal, 
+	    					 C_DI_REQUEST_NR, 
+						 META_VND_ANY)) ||
+	    /* RAD-ATR */
+	    !(ret->ds_rad_atr = meta_getspcbynr(m, C_DS_RAD_ATR)) ||
 	    !(ret->di_msg_auth = meta_getitembynr(m, ret->ds_rad_atr, 
 	    					 C_DI_MSG_AUTH, 
 						 META_VND_ANY)) ||
@@ -402,6 +418,14 @@ CONF *conf_new(META *m, char *basepath, char *modbasepath)
 	ret->inithead[VM_REP] = vm->head[VM_REP];
 	ret->inittail[VM_REP] = vm->tail[VM_REP];
 
+	/* Fixup REQ:Server-Pid, if given */
+
+	for(av = ret->inithead[VM_REQ]; av; av = av->next) {
+		if (av->i != ret->di_server_pid) continue;
+		av->l = getpid(); 
+		break; 
+	}
+
 	/* Free the vm and the fake interfaces, as they're not needed anymore, 
 	   and also close the configuration file. Note that the VM is not the
 	   owner of the request and reply lists. */
