@@ -48,6 +48,7 @@
  * 2002/10/02 - EvB - Added missing sys/time.h
  * 2003/06/06 - EvB - Added -2 flag to prevent binding as version 3, which is
  *		      now the default if the client library supports it
+ * 2005/06/21 - EvB - Set module interface version to 2
  */
 
 char radldap_id[] = "RADLDAP - Copyright (C) 2001 Emile van Bergen.";
@@ -225,12 +226,14 @@ ATRMAP *setup_atrmap()
 		/* Get first token, skip empty lines */
 		name = strtok(line, " \t\n\r"); if (!name) continue;
 		ret->ldap_atrs[ret->entries] = strdup(name);
+		ret->vnds[ret->entries] = C_VND_ANY;
 
 		/* Get other 3 tokens, convert to numeric and check syntax */
 		if (!(s = strtok(0, " \t\n\r"))	||
 		    (ret->spcs[ret->entries] = strtoul(s, &s, 10), s && *s) ||
 		    !(s = strtok(0, " \t\n\r"))	||
-		    (ret->vnds[ret->entries] = strtoul(s, &s, 10), s && *s) ||
+		    (*s != 'x' &&
+		     (ret->vnds[ret->entries] = strtoul(s, &s, 10), s && *s)) ||
 		    !(s = strtok(0, " \t\n\r"))	||
 		    (ret->atrs[ret->entries] = strtoul(s, &s, 10), s && *s)) {
 
@@ -325,8 +328,10 @@ LDAP *setup_ldap()
 U_INT32_T set_nak(U_INT32_T *msgbuf)
 {
 	msgbuf[1] = htonl(28);
-	msgbuf[2] = htonl(C_DS_INTERNAL); msgbuf[3] = 0;
-	msgbuf[4] = htonl(C_DI_INT); msgbuf[5] = htonl(4);
+	msgbuf[2] = htonl(C_DS_INTERNAL); 
+	msgbuf[3] = htonl(C_VND_ANY);
+	msgbuf[4] = htonl(C_DI_INT); 
+	msgbuf[5] = htonl(4);
 	msgbuf[6] = 0;
 
 	return 7 << 2;
@@ -335,12 +340,14 @@ U_INT32_T set_nak(U_INT32_T *msgbuf)
 
 /*
  * Set response message according to the LDAP search we perform and
- * return its size
+ * return the number of objects retrieved or -1 if error. The buffer
+ * passed in 'msgbuf' must be C_MAX_MSGSIZE long; the length of the
+ * response will be returned in the uint32 pointed to by n.
  */
 
 
-U_INT32_T do_search(LDAP *ld, char *basedn, char *filter, 
-		   ATRMAP *map, U_INT32_T *msgbuf)
+int do_search(LDAP *ld, char *basedn, char *filter, 
+	      ATRMAP *map, U_INT32_T *msgbuf, U_INT32_T *len)
 {
 	static char dbgbuf[MAPF_LINELEN];
 	struct timeval tv;
@@ -360,7 +367,7 @@ U_INT32_T do_search(LDAP *ld, char *basedn, char *filter,
 	n = ldap_search_st(ld, basedn, LDAP_SCOPE_SUBTREE, filter,
 			   map->ldap_atrs, 0, &tv, &res);
 	if (n == -1 || !res) { ldap_perror(ld, "radldap: Could not search"); 
-			       cnt = 0; goto srch_done; }
+			       cnt = -1; goto srch_done; }
 
 	/* Get first entry */
 	for(ent = ldap_first_entry(ld, res), cnt = 0; 
@@ -466,7 +473,7 @@ srch_done:
 
 	/* Add int attribute with found object count */
 	o[0] = htonl(C_DS_INTERNAL); 
-	o[1] = 0; 
+	o[1] = htonl(C_VND_ANY); 
 	o[2] = htonl(C_DI_INT); 
 	o[3] = htonl(4); 
 	o[4] = htonl(cnt);
@@ -475,8 +482,9 @@ srch_done:
 	/* Write size in msgbuf and return it */
 	m = (o - msgbuf) << 2;
 	msgbuf[1] = htonl(m);
+	if (len) *len = m;
 
-	return m;
+	return cnt;
 }
 
 
@@ -489,7 +497,7 @@ int main(int argc, char **argv)
 {
 	static U_INT32_T msgbuf[(C_MAX_MSGSIZE >> 2) + 1];
 	U_INT32_T spc, vnd, atr, len, *i, *e;
-	int len_cred[2], len_str[2];
+	int len_cred[2], len_str[2], res;
 	char *cred[2], *str[2];
 	ATRMAP *map;
 	LDAP *ld;
@@ -509,11 +517,12 @@ int main(int argc, char **argv)
 
 		/* Read header */
 		if (read(0, msgbuf, 8) != 8) { perror("radldap: read"); break; }
-		if (ntohl(msgbuf[0]) != 0xdeadbeef) {
+		if (ntohl(msgbuf[0]) != 0xbeefdead) {
 			fprintf(stderr, "radldap: Invalid magic 0x%08x!\n", 
 				ntohl(msgbuf[0])); 
 			break;
 		}
+		msgbuf[0] = htonl(0xdeadbeef);
 		len = ntohl(msgbuf[1]);
 		if (len < 8 || len > sizeof(msgbuf) - 4) {
 			fprintf(stderr, "radldap: Invalid length %d!\n", len); 
@@ -526,6 +535,9 @@ int main(int argc, char **argv)
 			break;
 		}
 
+		/* Default LDAP status if we don't get to searching */
+		res = 0;
+
 		/*
 		 * Loop through the attributes. We're interested in the last
 		 * instance of 'User-Name' (space C_DS_RAD_ATR, vendor 0, attr.
@@ -548,14 +560,15 @@ int main(int argc, char **argv)
 					spc, vnd, atr, len);
 			}
 
-			if (spc == C_DS_RAD_ATR && vnd == 0 &&
+			if (spc == C_DS_RAD_ATR && vnd == C_VND_ANY &&
 			    (atr == C_DI_USER_NAME ||
 			     atr == C_DI_USER_PASSWORD)) {
 
 				cred[atr == C_DI_USER_PASSWORD] = (char *)(i+4);
 				len_cred[atr == C_DI_USER_PASSWORD] = len;
 			}
-			else if (spc == C_DS_INTERNAL && vnd == 0 &&
+			else if (spc == C_DS_INTERNAL && 
+				 vnd == C_VND_ANY &&
 				 atr == C_DI_STR) {
 
 				str[0] = str[1]; len_str[0] = len_str[1];
@@ -579,7 +592,8 @@ int main(int argc, char **argv)
 			if (do_bind(ld, cred[0], cred[1])) {
 				if (debug) ldap_perror(ld, "radldap: Bind "
 							   "failed");
-				len = set_nak(msgbuf); goto reply;
+				len = set_nak(msgbuf); 
+				goto reply;
 			}
 		}
 
@@ -599,9 +613,10 @@ int main(int argc, char **argv)
 		 * Perform subtree search under the base dn specified in the 
 		 * first 'str' using the filter specified in the second and
 		 * translate the returned attributes using our mapping table.
+		 * If there's a serious LDAP error, we rebind after the reply.
 		 */
 
-		len = do_search(ld, str[0], str[1], map, msgbuf);
+		res = do_search(ld, str[0], str[1], map, msgbuf, &len);
 		
 		/*
 		 * Send a reply with size 'len'; unbind/rebind if that was asked
@@ -609,7 +624,7 @@ int main(int argc, char **argv)
 reply:
 		if (write(1, msgbuf, len) != len) { perror("radldap: write"); 
 						    break; }
-		if (unbind_after_req) {
+		if (res < 0 || unbind_after_req) {
 			signal(SIGPIPE, SIG_IGN);
 			ldap_unbind_s(ld);
 			signal(SIGPIPE, SIG_DFL);
